import datetime
import inspect
import logging
from collections.abc import Sequence
from pathlib import Path

import requests
import requests_cache

from . import data, fetchers, notification, util

logger = logging.getLogger(__name__)

MIN_COOKIES_THRESHOLD = 10

try:
    import cloudscraper
except ImportError:
    cloudscraper = None
    logger.exception('cloudscraper not installed: Certain fetchers might not work')

try:
    import browser_cookie3
except ImportError:
    logger.exception('browser_cookie3 not installed: Certain fetchers might not work')
    browser_cookie3 = None


OUTPUT_PATH = Path('./output')


STANDARD_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:122.0) Gecko/20100101 Firefox/122.0',
    'Accept-Language': 'en-US,en;q=0.5',
    'Accept-Encoding': 'gzip, deflate, br',
}


if cloudscraper:

    class CachedCfScrape(requests_cache.CacheMixin, cloudscraper.CloudScraper):
        pass


CACHE_EXPIRE_DEFAULT = datetime.timedelta(days=7)


def get_session(
    cookiejar: Sequence,
    *,
    with_cfscrape: bool,
    ignore_cache: bool,
) -> requests.Session:
    session_class = requests_cache.CachedSession
    if with_cfscrape:
        if cloudscraper:
            session_class = CachedCfScrape

            if ignore_cache:
                logger.warning('HTTP cache disabled')
                return cloudscraper.create_scraper(
                    interpreter='js2py',
                    delay=5,
                    debug=False,
                )
        else:
            logger.error('Expected cloudscraper, but not defined!')
    elif ignore_cache:
            logger.warning('HTTP cache disabled')
            return requests.Session()
    session = session_class(
        OUTPUT_PATH / 'web_cache',
        cookies=cookiejar,
        expire_after=CACHE_EXPIRE_DEFAULT,
    )
    for cookie in cookiejar:
        session.cookies.set_cookie(cookie)
    return session


def available_scrapers() -> list[type[data.Scraper]]:
    fetchers.load_fetcher_modules()
    subclasses = []
    class_queue = [data.Scraper]
    while class_queue:
        clazz = class_queue.pop()
        if inspect.isabstract(clazz):
            class_queue.extend(clazz.__subclasses__())
        else:
            subclasses.append(clazz)
    return subclasses


def available_scraper_names() -> list[str]:
    return [scraper_cls.__name__ for scraper_cls in available_scrapers()]


def get_cookiejar(use_cookiejar: bool):
    if use_cookiejar:
        logger.warning('Got cookiejar from firefox')
        cookiejar = browser_cookie3.firefox()
        if len(cookiejar) > MIN_COOKIES_THRESHOLD:
            return cookiejar
        browser_cookie3.firefox(
            '/home/jmaa/.cachy/mbui5xg7.default-release/cookies.sqlite',
        )
        logger.warning('Cookiejar has %s cookies', len(cookiejar))
        if len(cookiejar) > MIN_COOKIES_THRESHOLD:
            return cookiejar
    logger.warning('No cookiejar is used')
    return []


def main(
    scraper_filter: frozenset[str],
    *,
    use_cookiejar: bool,
    ignore_cache: bool,
    notification_types: frozenset[notification.NotificationType],
) -> None:
    cookiejar = get_cookiejar(use_cookiejar)

    if len(notification_types) == 0:
        logger.info('No notifications enabled: Notifications will not be sent!')

    for scraper_cls in available_scrapers():
        if scraper_cls.__name__ not in scraper_filter:
            continue
        session = get_session(
            cookiejar=cookiejar,
            with_cfscrape=scraper_cls.requires_cfscrape(),
            ignore_cache=ignore_cache,
        )
        scraper = scraper_cls(session)
        logger.info(
            'Running %s (%s), appending to "%s"',
            scraper_cls.__name__,
            type(session).__name__,
            scraper.dataset_name,
        )
        result_rows = []
        try:
            for result in scraper.scrape():
                result_rows.append(result)
                del result
        except requests.exceptions.HTTPError as e:
            logger.exception('Failed in running %s', scraper_cls.__name__)
            logger.exception('User-Agent: %s', e.request.headers['user-agent'])
            continue
        status = util.extend_csv_file(
            OUTPUT_PATH / f'{scraper.dataset_name}.csv',
            result_rows,
            deduplicate_mode=scraper.deduplicate_mode,
            deduplicate_ignore_columns=scraper.deduplicate_ignore_columns,
        )
        logger.info('Scraper done: %s', scraper.dataset_name)

        if status['extended']:
            notification.send_notifications(
                session,
                scraper_cls.__name__,
                status['dicts'][-1],
                notification_types,
            )

        del scraper, session
